@@ -1,14 +1,18 @@
 package Net::DNS::Question;
 #
-# $Id: Question.pm 264 2005-04-06 09:16:15Z olaf $
+# $Id: Question.pm 388 2005-06-22 10:06:05Z olaf $
 #
 use strict;
+BEGIN { 
+    eval { require bytes; }
+} 
+
 use vars qw($VERSION $AUTOLOAD);
 
 use Carp;
 use Net::DNS;
 
-$VERSION = (qw$LastChangedRevision: 264 $)[1];
+$VERSION = (qw$LastChangedRevision: 388 $)[1];
 
 =head1 NAME
 
